/*
 * Copyright 2020 Kunbus GmbH.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.kunbus.revpi;

public class PiControl {

    static final String PICONTROL_DEVICE = "/dev/piControl0";

// The config file moved to /etc/revpi.
// If it cannot be found there, the old location should be used.
    static final String PICONFIG_FILE = "/etc/revpi/config.rsc";
    static final String PICONFIG_FILE_WHEEZY = "/opt/KUNBUS/config.rsc";

    static final int REV_PI_DEV_FIRST_RIGHT = 32;	// address of first module on the right side of the RevPi Core
    static final int REV_PI_DEV_CNT_MAX = 64;	// max. number of

// Module Id
// 0x0001 - 0x3000      KUNBUS Modules (e.g. DIO, Gateways, ...)
// 0x3001 - 0x7000      not used
// 0x6001 - 0x7000      KUNBUS Software Adapters
// 0x7001 - 0x8000      User defined Software Adapters
// 0x8001 - 0xb000      KUNBUS Modules configured but not connected
// 0xb001 - 0xffff      not used
    static final int PICONTROL_SW_OFFSET = 0x6001;      // 24577
    static final int PICONTROL_SW_MODBUS_TCP_SLAVE = 0x6001;     // 24577
    static final int PICONTROL_SW_MODBUS_RTU_SLAVE = 0x6002;     // 24578
    static final int PICONTROL_SW_MODBUS_TCP_MASTER = 0x6003;     // 24579
    static final int PICONTROL_SW_MODBUS_RTU_MASTER = 0x6004;     // 24580
    static final int PICONTROL_SW_PROFINET_CONTROLLER = 0x6005;     // 24581
    static final int PICONTROL_SW_PROFINET_DEVICE = 0x6006;     // 24582

    static final int PICONTROL_NOT_CONNECTED = 0x8000;
    static final int PICONTROL_NOT_CONNECTED_MASK = 0x7fff;

    static final int PICONTROL_USER_MODULE_TYPE = 0x8000;  // old definition, will be removed soon
    static final int PICONTROL_USER_MODULE_MASK = 0x7fff;  // old definition, will be removed soon

    static final int PICONTROL_CONFIG_ERROR_WRONG_MODULE_TYPE = -10;
    static final int PICONTROL_CONFIG_ERROR_WRONG_INPUT_LENGTH = -11;
    static final int PICONTROL_CONFIG_ERROR_WRONG_OUTPUT_LENGTH = -12;
    static final int PICONTROL_CONFIG_ERROR_WRONG_CONFIG_LENGTH = -13;
    static final int PICONTROL_CONFIG_ERROR_WRONG_INPUT_OFFSET = -14;
    static final int PICONTROL_CONFIG_ERROR_WRONG_OUTPUT_OFFSET = -15;
    static final int PICONTROL_CONFIG_ERROR_WRONG_CONFIG_OFFSET = -16;

    static final int PICONTROL_STATUS_RUNNING = 0x01;
    static final int PICONTROL_STATUS_EXTRA_MODULE = 0x02;
    static final int PICONTROL_STATUS_MISSING_MODULE = 0x04;
    static final int PICONTROL_STATUS_SIZE_MISMATCH = 0x08;
    static final int PICONTROL_STATUS_LEFT_GATEWAY = 0x10;
    static final int PICONTROL_STATUS_RIGHT_GATEWAY = 0x20;

    static final int PICONTROL_LED_A1_GREEN = 0x01;
    static final int PICONTROL_LED_A1_RED = 0x02;
    static final int PICONTROL_LED_A2_GREEN = 0x04;
    static final int PICONTROL_LED_A2_RED = 0x08;

    static {
        System.loadLibrary("PiControlIfWrapper");
    }

    public PiControl() {
        init();
    }

    /**
     * @brief Open Pi Control Interface
     *
     * Initialize the Pi Control Interface
     */
    private native void init();

    /**
     * @brief Reset Pi Control Interface
     *
     * Initialize the Pi Control Interface
     */
    public native int reset();

    /**
     *
     * @brief Get Processdata
     *
     * Gets Processdata from a specific position
     *
     * @param offset
     * @param length
     * @param data
     *
     * @return Number of Bytes read or negative on error
     *
     */
    public native int read(long offset, long length, Long data);

    /**
     * @brief Set Processdata
     *
     * Writes Processdata at a specific position
     *
     * @param offset
     * @param length
     * @param data
     *
     * @return Number of Bytes written or negative on error
     *
     */
    public native int write(long offset, long length, long data);

    /**
     * @brief Get Device Info
     *
     * Get Description of connected devices.
     *
     * @param dev An array of 20 entries of type SDeviceInfo.
     *
     * @return Number of detected devices or negative on error
     *
     */
    //public native int getDeviceInfo(SDeviceInfo[] dev);
    
    public native int getDeviceInfoList(SDeviceInfo[] dev);

    /**
     * @brief Get Bit Value
     *
     * Get the value of one bit in the process image.
     *
     * @param spiValue
     *
     * @return 0 on success or negative on error
     *
     */
    public native int getBitValue(SPIValue spiValue);

    /**
     * @brief Set Bit Value
     *
     * Set the value of one bit in the process image.
     *
     * @param spiValue Pointer to SPIValue.
     *
     * @return 0 on success or negative on error
     *
     */
    public native int setBitValue(SPIValue spiValue);

    /**
     *
     * @brief Get Variable Info
     *
     * Get the info for a variable.
     *
     * @param spiVariable Pointer to SPIVariable.
     *
     * @return 0 on success or negative on error
     *
     */
    public native int getVariableInfo(SPIVariable spiVariable);

    /**
     *
     * @brief Get Variable offset by name
     *
     * Get the offset of a variable in the process image. This does NOT work for
     * variable of type bool.
     *
     * @param name Name of the variable
     *
     * @return >= 0 offset < 0 in case of error
     */
    public native int findVariable(String name);
}
